# ---------------------------------------------------------------------

# Article:       W.K. Hrdle, M. Osipenko

#		"Spatial risk premia on temperature and hedging weather exposure in electricity"

# ---------------------------------------------------------------------

# Quantlet:    	main.R, city.R, pricehat.R, pca.R, rp.R

# ---------------------------------------------------------------------

# Description: 	This is the application of functional PCA to risk premia on temperature derivatives from CMA

# ---------------------------------------------------------------------

# Usage:       	this file is called by main.R. Please modify your working directory

# ---------------------------------------------------------------------

# Inputs:      	temperature data and prices from CME in .csv 

# ---------------------------------------------------------------------

# Output:      	Plots and Estimation results shown in the paper

# ---------------------------------------------------------------------

# Example:     	-

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------
graphics.off()
rm(list=ls(all=TRUE))
setwd("C:/.../Quantlet")# working directory
tabletemp<-c("AmsterdamTemp.csv", "BarcelonaTemp.csv", "BerlinTemp.csv", "EssenTemp.csv",
"LondonTemp.csv", "MadridTemp.csv", "ParisTemp.csv", "RomeTemp.csv","StockholmTemp.csv","LeipzigTemp.csv")
tablename<-c("tempforA.csv", "tempforBc.csv", "tempforB.csv", "tempforE.csv",
"tempforL.csv", "tempforM.csv", "tempforP.csv", "tempforR.csv","tempforS.csv","tempforLei.csv")
csvname<-c("Amsterdam.csv", "Barcelona.csv", "Berlin.csv", "Essen.csv",
"London.csv", "Madrid.csv", "Paris.csv", "Rome.csv","Stockholm.csv","Leipzig.csv")
plotnames<-c("sa.pdf","sbc.pdf", "sb.pdf", "se.pdf", "sl.pdf", "sm.pdf", "sp.pdf",
"sr.pdf","ss.pdf","sle.pdf")
plotnamev<-c("va.pdf","vbc.pdf", "vb.pdf", "ve.pdf", "vl.pdf", "vm.pdf", "vp.pdf",
"vr.pdf","vs.pdf","vle.pdf")
cityname<-c("Amsterdam", "Barcelona", "Berlin", "Essen",
"London", "Madrid", "Paris", "Rome","Stockholm","Leipzig")
cc<-c("ca.csv","cbc.csv", "cb.csv","ce.csv", "cl.csv", "cm.csv", "cp.csv","cr.csv","cs.csv","clei.csv")

for (li in 1:length(tabletemp))
{
	setwd(.../Quantlet/data")
	#reading weather data
	weather<-read.csv(tabletemp[li], sep=";",dec=",",header=FALSE)
	#define temp as temperature
	temp<-weather[,3]
	time<-c(1:length(temp))
################################################################################################
	# fit least squares, number of terms selection by BIC
	X<-cbind(sin(2*pi*(time)/365),cos(2*pi*(time)/365),sin(4*pi*(time)/365),cos(4*pi*(time)/365),
	sin(6*pi*(time)/365),cos(6*pi*(time)/365),sin(8*pi*(time)/365),cos(8*pi*(time)/365))
	crit<-matrix(NA,ncol(X))
	for (l in 1:ncol(X))
		{
 		fit.season<-lm(temp~time+X[,1:l])
		crit[l]<-AIC(fit.season, k=log(nrow(X)))
		}
	s=which(crit==min(crit))
	fit.season<-lm(temp~time+X[,1:s])
	sa<-summary(fit.season)
	sa<-data.frame(coef(sa))
	
	print(cityname[li])
	print("seasonality")
	print(sa)
	fit.hat.season<-fitted(fit.season)
	res=temp-fit.hat.season
#################################################################################################
	#define fitted values and plot last observations
	install.packages("TSA")	
	install.packages("tseries")	
	library(TSA)
	library(tseries)
	adf.test(res)# test for stationarity of deseasonolized temperature
	kpss.test(res,lshort=FALSE)
	kpss.test(res,null=c('Trend'),lshort=FALSE)
################################################################################
	#fit AR(p) model
	#q=c(0,1)
	p=c(1,2,3,4,5)
	sbc=matrix(0,length(p),1)
	for (j in 1:length(p)){
	fit.resid.AR<-arima(res, order=c(p[j],0,0), include.mean=FALSE,method='ML')
	sbc[j]=-2*fit.resid.AR$loglik+(p[j])*log(length(res))#compute SBC
	}
	ind<-which(sbc==min(sbc),arr.ind = TRUE)#find max sbc
 	p<-ind[1,1]
	fit.resid.AR<-arima(res, order=c(p,0,0), include.mean=FALSE,method='CSS-ML')
	coef<-fit.resid.AR$coef
	t.Stat<-fit.resid.AR$coef/sqrt(diag(fit.resid.AR$var.coef))
	p.value<-2*(1-pt(q = abs(t.Stat), df=length(temp)-3))
 	da<-data.frame(round(coef,3),round(t.Stat,3),round( p.value,5))
	print("AR process")
	print(da)
################################################################################
	#define residuals and check
	fit.hat.AR<-fitted(fit.resid.AR)
 	res2<-fit.resid.AR$resid
	x=seq(0,3650,365*2)
#########################################################################
	setwd(".../Quantlet/figures")
	pdf(file=plotnames[li],paper="special",width=6, height=6)
	plot(temp[(length(temp)-3650):length(temp)],col="dimgray",ylab="",
	xlab="",type='l',main=cityname[li],xaxt="n",yaxt="n",cex.main=1.4,font.main=1,ylim=c(-20,30))
	lines(fit.hat.season[(length(temp)-3650):length(temp)],lwd=4)
	axis(1, x, labels = c("2000","2002","2004","2006","2008","2010"),lwd=1,cex.axis=1.3) 
	axis(2,lwd=1,cex.axis=1.3)
	dev.off()
	
#####################################################################################
	#define seasonal variance
	sigma<-numeric(365)
	#number of groups to compute the seasonal variation
	nd=round(length(temp)/365+1)
	resid.day<-matrix(NA,365,nd)
 	for(i in 1:365)
  		{
    		for (k in 1:nd)
     			{
        		ifelse((i+((k-1)*365))<=length(temp),resid.day[i,k]<-res2[i+((k-1)*365)],resid.day[i,k]<-NA)
     			}
        		sigma[i]<-(sd(resid.day[i,],na.rm=TRUE))^2
  		}
#################################################################################################################
	# fit least squares to the variances, BIC
	t=seq(1,365,1)
	X<-cbind(sin(2*pi*(t)/365),cos(2*pi*(t)/365),sin(4*pi*(t)/365),cos(4*pi*(t)/365),
	sin(6*pi*(t)/365),cos(6*pi*(t)/365),sin(8*pi*(t)/365),cos(8*pi*(t)/365))
	crit<-matrix(NA,ncol(X))
	for (l in 1:ncol(X)){
 	fit.var<-lm(sigma~X[,1:l])
	crit[l]<-AIC(fit.var, k=log(nrow(X)))}
	s=which(crit==min(crit))
	fit.var<-lm(sigma~X[,1:s])
	sav<-summary(fit.var)
	sav<-data.frame(coef(sav))
	print("seasonal volatility")
	print(sav)
	fit.hat.var<-fitted(fit.var)
	
	
	a0=c(91,121,152,182,213,244,274)
	b0=c(120,151,181,212,243,273,303)

	tt<-cbind(seq(a0[1],b0[1],length.out=100), seq(a0[2],b0[2],length.out=100),
		seq(a0[3],b0[3],length.out=100), seq(a0[4],b0[4],length.out=100),
		seq(a0[5],b0[5],length.out=100), seq(a0[6],b0[6],length.out=100),
		seq(a0[7],b0[7],length.out=100))
	varf<-matrix(NA,nrow(tt),ncol(tt))
	for (il in 1:ncol(tt))
	{
		XX<-cbind(sin(2*pi*(tt[,il])/365),cos(2*pi*(tt[,il])/365),sin(4*pi*(tt[,il])/365),cos(4*pi*(tt[,il])/365),
		sin(6*pi*(tt[,il])/365),cos(6*pi*(tt[,il])/365),sin(8*pi*(tt[,il])/365),cos(8*pi*(tt[,il])/365))
		varf[,il]<-sav$Estimate[1]+XX[,1:s]%*%sav$Estimate[2:(s+1)]
	}
	

	
	pdf(file=plotnamev[li],paper="special",width=6, height=6)
 	plot(sigma,col="dimgray",ylab="",
	xlab="",type='l',main=cityname[li],cex.main=1.4,font.main=1,xaxt="n",yaxt="n",ylim=c(0,10))
 	lines(fit.hat.var,lwd=4)
	x=c(1,32,61,92,122,153,183,214,245,275,306,336,365)
	axis(1, x, labels = c("Jan","","Mar","","May","","Jul","","Sep","","Nov","","Jan"),lwd=1,cex.axis=1.2) 
	axis(2,lwd=1,cex.axis=1.3) 
	dev.off()
	
##########################################################################################################################3
	#test standardized residuals
 	s<-rep(fit.hat.var,nd+1)
	sigmas<-s[1:length(temp)]
	resid.std<-res2/sqrt(sigmas)
	z<-rnorm(100000,0,sd(resid.std,na.rm=TRUE))
	install.packages("moments")
	library(moments)
	skewness(resid.std,na.rm=TRUE)
	kurtosis(resid.std,na.rm=TRUE)
	resid.std.jb<-resid.std[(3+1):length(resid.std)] #to omit NA values in residuals
	jarque.bera.test(resid.std.jb) 
	tempfor<-cbind(weather[,2],temp,fit.hat.season,fit.hat.AR)
	setwd("C:/Users/maria.osipenko/Documents/FPCAinR/Quantlet/output")
	write.csv(tempfor,file=tablename[li])
	write.csv(sigma,file=csvname[li])
	write.csv(coef(fit.resid.AR),file=cc[li])
}

