# ---------------------------------------------------------------------

# Article:       W.K. Hrdle, M. Osipenko

#		"Spatial risk premia on temperature and hedging weather exposure in electricity"

# ---------------------------------------------------------------------

# Quantlet:    	main.R, city.R, pricehat.R, pca.R, rp.R

# ---------------------------------------------------------------------

# Description: 	This is the application of functional PCA to risk premia on temperature derivatives from CMA

# ---------------------------------------------------------------------

# Usage:       	this file is called by main.R. Please modify your working directory

# ---------------------------------------------------------------------

# Inputs:      	temperature data and prices from CME in .csv 

# ---------------------------------------------------------------------

# Output:      	Plots and Estimation results shown in the paper

# ---------------------------------------------------------------------

# Example:     	-

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------
graphics.off()
rm(list=ls(all=TRUE))
#
setwd('C:/.../Quantlet/output')#delete the old scores at first
install.packages("fda")
library(fda)
a<-read.csv("Amsterdam.csv")
bc<-read.csv("Barcelona.csv")
b<-read.csv("Berlin.csv")
e<-read.csv("Essen.csv")
l<-read.csv("London.csv")
m<-read.csv("Madrid.csv")
p<-read.csv("Paris.csv")
r<-read.csv("Rome.csv")
s<-read.csv("Stockholm.csv")
le<-read.csv("Leipzig.csv")
ams=sqrt(a[,2])
bac=sqrt(bc[,2])
ber=sqrt(b[,2])
ess=sqrt(e[,2])
lon=sqrt(l[,2])
mad=sqrt(m[,2])
par=sqrt(p[,2])
rom=sqrt(r[,2])
sto=sqrt(s[,2])
lei=sqrt(le[,2])
setwd('C:/.../Quantlet/output/scores')
filename=c("pcaa.pdf","pcabc.pdf", "pcab.pdf", "pcae.pdf", "pcal.pdf", "pcam.pdf", "pcap.pdf", "pcar.pdf", "pcas.pdf","pcale.pdf")
name=c("Amsterdam","Barcelona","Berlin","Essen","London","Madrid","Paris","Rome","Stockholm","Leipzig")
fname<-c("Amsterdam.csv","Barcelona.csv","Berlin.csv","Essen.csv","London.csv","Madrid.csv","Paris.csv","Rome.csv","Stockholm.csv","Leipzig.csv")
volas<-cbind(ams,bac,ber,ess,lon,mad,par,rom,sto,lei)

for (i in 1:length(name))
{
	
	a0=c(91,121,152,182,213,244,274)
	b0=c(120,150,181,211,242,273,303)
	vola=cbind(volas[a0[1]:b0[1],i],volas[a0[2]:b0[2],i],volas[a0[3]:b0[3],i],
	volas[a0[4]:b0[4],i],volas[a0[5]:b0[5],i],volas[a0[6]:b0[6],i],volas[a0[7]:b0[7],i] )
	daybasis <- create.fourier.basis(c(1, 30), nbasis=5)
	

	volafd<-data2fd(vola,seq(1, 30,1),daybasis)
	pcas<-pca.fd(volafd,3)
	#
	setwd('C:/.../Quantlet/figures')
	pdf(file=filename[i],paper="special",width=6, height=6)
	plot(pcas$harmonics,lwd=4,col="black",main=name[i],cex.main=1.3,font.main=1,ylab="",
	ylim=c(-0.6,0.6),xaxt="n",yaxt="n")
	axis(1, lwd=1,cex.axis=1.3) 
	axis(2,lwd=1,cex.axis=1.3)
	dev.off()
	#
	setwd('C:/.../Quantlet/output/scores')
	write.csv(pcas$scores,file=fname[i])
	print(filename[i])
	print(pcas$scores)
}
