# ---------------------------------------------------------------------

# Article:       W.K. Hrdle, M. Osipenko

#		"Spatial risk premia on temperature and hedging weather exposure in electricity"

# ---------------------------------------------------------------------

# Quantlet:    	main.R, city.R, pricehat.R, pca.R, rp.R

# ---------------------------------------------------------------------

# Description: 	This is the application of functional PCA to risk premia on temperature derivatives from CMA

# ---------------------------------------------------------------------

# Usage:       	this file is called by main.R. Please modify your working directory

# ---------------------------------------------------------------------

# Inputs:      	temperature data and prices from CME in .csv 

# ---------------------------------------------------------------------

# Output:      	Plots and Estimation results shown in the paper

# ---------------------------------------------------------------------

# Example:     	-

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------
graphics.off()
rm(list=ls(all=TRUE))



install.packages("msr")
library(msm)

dates1=(array( c(
    20040401, 20040501, 20040601, 20040701, 20040801, 20040901, 20041001,
    20050401, 20050501, 20050601, 20050701, 20050801, 20050901, 20051001,
    20060401, 20060501, 20060601, 20060701, 20060801, 20060901, 20061001,
    20070401, 20070501, 20070601, 20070701, 20070801, 20070901, 20071001,
    20080401, 20080501, 20080601, 20080701, 20080801, 20080901, 20081001,
    20090401, 20090501, 20090601, 20090701, 20090801, 20090901, 20091001,
    20100401, 20100501, 20100601, 20100701, 20100801, 20100901, 20101001),dim=c(7,7)))
dates2=(array( c(
    20040430, 20040531, 20040630, 20040731, 20040831, 20040930, 20041031,
    20050430, 20050531, 20050630, 20050731, 20050831, 20050930, 20051031,
    20060430, 20060531, 20060630, 20060731, 20060831, 20060930, 20061031,
    20070430, 20070531, 20070630, 20070731, 20070831, 20070930, 20071031,
    20080430, 20080531, 20080630, 20080731, 20080831, 20080930, 20081031,
    20090430, 20090531, 20090630, 20090731, 20090831, 20090930, 20091031,
    20100430, 20100531, 20100630, 20100731, 20100831, 20100930, 20101031),dim=c(7,7)))
tablename<-c("priceA.csv","priceBc.csv", "priceB.csv", "priceE.csv", "priceL.csv",
"priceM.csv", "priceP.csv", "priceR.csv", "priceS.csv")
tabletemp<-c("tempforA.csv", "tempforBc.csv", "tempforB.csv", "tempforE.csv",
"tempforL.csv", "tempforM.csv", "tempforP.csv", "tempforR.csv","tempforS.csv")
filename<-c("rpma.csv", "rpmbc.csv", "rpmb.csv", "rpme.csv", "rpml.csv",
"rpmm.csv", "rpmp.csv", "rpmr.csv", "rpms.csv")
cc<-c("ca.csv","cbc.csv", "cb.csv","ce.csv", "cl.csv", "cm.csv", "cp.csv","cr.csv","cs.csv")
cityname<-c("Amsterdam","Barcelona","Berlin","Essen","London","Madrid","Paris","Rome","Stockholm")
price.hat=matrix(NA,nrow(dates1),ncol(dates1))
dimnames(price.hat)<-list(month.abb[4:10], 2004:2010)


for (k in 1:length(tablename))
{	setwd('C:/.../Quantlet/output')
	Aest<-read.csv(cc[k])
	Aest<-Aest[,2]
	if (length(Aest)==4)
	{
	alpha1=4-Aest[1];alpha2=3*alpha1-Aest[2]-6;
	alpha3=-3*alpha1+4+2*alpha2-Aest[3];alpha4=alpha1-alpha2+alpha3-Aest[4];
	A=cbind(c(0,0,0,-alpha4),c(1,0,0,-alpha3),c(0,1,0,-alpha2),c(0,0,1,-alpha1))
	ip<-diag(1,4,4);
	ep=c(0, 0, 0, 1);
	e1=c(1, 0, 0, 0);
	}
	if (length(Aest)==3)
	{
	alpha1=3-Aest[1];alpha2=2*alpha1-Aest[2]-3;alpha3=alpha2+1-alpha1-Aest[3];
	A=cbind(c(0,0,-alpha3),c(1,0,-alpha2),c(0,1,-alpha1)) 
	ip<-diag(1,3,3);
	ep=c(0, 0, 1);
	e1=c(1, 0, 0);
	}
	tempfor<-read.csv(tabletemp[k])
	setwd('C:/.../Quantlet/data/')
	price<-read.csv(tablename[k],sep=";", dec=",",header=F)
	tempfor<-tempfor[,2:5]
	rp<-matrix(NA,nrow(dates1),ncol(dates1))
	
	for (i in 1:ncol(dates1))
	{
		for (j in 1:nrow(dates1))
		{
		beg=dates1[j,i]
		en=dates2[j,i]
		it=which(tempfor[,1]==beg)
		itau1=it
		itau2=which(tempfor[,1]==en)
		if (length(itau2)==0){price.hat[j,i]=NA} else
 		{season=sum(tempfor[itau1:itau2, 3])
			if (ncol(A)==3)
			{
			x=c(tempfor[it-3,4], tempfor[it-2,4], tempfor[it-1,4])
			}
			if (ncol(A)==4)
			{
			x=c(tempfor[it-4,4], tempfor[it-3,4], tempfor[it-2,4], tempfor[it-1,4])
			}
			if (ncol(A)==7)
			{
			x=c(tempfor[it-7,4], tempfor[it-6,4], tempfor[it-5,4], tempfor[it-4,4], tempfor[it-3,4],
			tempfor[it-2,4], tempfor[it-1,4])
			}
			ax=e1%*%solve(A)%*%(MatrixExp(A*(itau2-itau1+1))-diag(1,nrow(A),ncol(A)))%*%x
			price.hat[j,i]=season+ax
		}
		if (is.na(price[j,i])==FALSE){rp[j,i]<-price[j,i]-price.hat[j,i]}
		}
	}
	rpma=matrix(NA,7,1)
	for (i in 1: nrow(rp))
	{
	rpx=rp[i,]
	rpx=rpx[is.na(rpx)==FALSE]
	rpx=mean(rpx)
	rpma[i]=rpx
	}
 	dimnames(rpma)<-list(month.abb[4:10],"rp")

	setwd('C:/.../Quantlet/output')
	write.csv(rpma,file=filename[k])
	print(cityname[k])
	print("price with mpr=0")
	print(price.hat)
	print("risk premia")
	print(rpma)
 	
}
