# ---------------------------------------------------------------------

# Article:       W.K. Hrdle, M. Osipenko

#		"Spatial risk premia on temperature and hedging weather exposure in electricity"

# ---------------------------------------------------------------------

# Quantlet:    	main.R, city.R, pricehat.R, pca.R, rp.R

# ---------------------------------------------------------------------

# Description: 	This is the application of functional PCA to risk premia on temperature derivatives from CMA

# ---------------------------------------------------------------------

# Usage:       	this file is called by main.R. Please modify your working directory

# ---------------------------------------------------------------------

# Inputs:      	temperature data and prices from CME in .csv 

# ---------------------------------------------------------------------

# Output:      	Plots and Estimation results shown in the paper

# ---------------------------------------------------------------------

# Example:     	-

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------
graphics.off()
rm(list=ls(all=TRUE))
setwd('C:/.../Quantlet/output')

rph<-read.csv('rpma.csv')
rphat<-rph
rph<-read.csv('rpmbc.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpmb.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpme.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpml.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpmm.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpmp.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpmr.csv')
rphat<-rbind(rphat,rph)
rph<-read.csv('rpms.csv')
rphat<-rbind(rphat,rph)
rpvec<-rphat[,2]
write.csv(rpvec,file="rpvec.csv")


scored<-read.csv("scores/Amsterdam.csv")
scores<-scored
scored<-read.csv("scores/Barcelona.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Berlin.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Essen.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/London.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Madrid.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Paris.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Rome.csv")
scores<-rbind(scores,scored)
scored<-read.csv("scores/Stockholm.csv")
scores<-rbind(scores,scored)
library(spdep)
library(spgwr)
lat<-c(rep(52.3,times=7),rep(41.28,times=7),rep(52.52,times=7),
rep(51.45,times=7),rep(51.48,times=7),rep(40.45,times=7),
rep(48.73,times=7),rep(41.78,times=7),rep(59.35,times=7))

lon<-c(rep(4.77,times=7),rep(2.07,times=7),rep(13.42,times=7),
rep(7.02,times=7),rep(-0.45,times=7),rep(-3.55,times=7),
rep(2.4,times=7),rep(12.58,times=7),rep(17.95,times=7))

dummy1<-matrix(0,length(rpvec),1)
dummy2<-matrix(0,length(rpvec),1)
dummy1[1:7]<-matrix(1,7,1)#Amsterdam 
dummy2[8:14]<-matrix(1,7,1)#Barcelona
dummy1[29:35]<-matrix(1,7,1)#London
dummy2[50:56]<-matrix(1,7,1)#Rome

dummy1<-(dummy1==1)
dummy2<-(dummy2==1)
#bw by cv
bw<-gwr.sel(rpvec~scores$V1+scores$V2+scores$V3+dummy1+dummy2,data=cbind(rpvec,scores[,2:4],dummy1,dummy2),
coords=cbind(lat,lon), method="cv")
gwrrp<-gwr(rpvec~scores$V1+scores$V2+scores$V3+dummy1+dummy2,data=cbind(rpvec,scores[,2:4]),
coords=cbind(lat,lon),bandwidth=bw,predict=TRUE,se.fit=TRUE,hatmatrix=TRUE,gweight=gwr.Gauss)
print(gwrrp)
rp<-gwrrp$SDF[[14]]#predicted risk premia
beta_0=unique(gwrrp$SDF[[2]])
beta_1=unique(gwrrp$SDF[[3]])
beta_2=unique(gwrrp$SDF[[4]])
beta_3=unique(gwrrp$SDF[[5]])
beta_4=unique(gwrrp$SDF[[6]])
beta_5=unique(gwrrp$SDF[[7]])
r2=unique(gwrrp$SDF$localR2)
cityname<-c("Amsterdam","Barcelona","Berlin","Essen","London","Madrid","Paris","Rome","Stockholm")
results=data.frame(cityname,beta_0,beta_1,beta_2,beta_3,beta_4,beta_5,r2)
print(results)
BFC02.gwr.test(gwrrp)#s.91-92 fotheringham
R2=sum((gwrrp$SDF$pred-mean(gwrrp$SDF$pred))^2)/(sum((gwrrp$SDF$pred-mean(gwrrp$SDF$pred))^2)+sum((rpvec-gwrrp$SDF$pred)^2))
R2
############################################################
#par(mfrow=c(1,1))
setwd("C:/.../Quantlet/figures/")
filename=c("ga.pdf","gbc.pdf", "gb.pdf", "ge.pdf", "gl.pdf", "gm.pdf", "gp.pdf", "gr.pdf", "gs.pdf")
for (i in 1:length(cityname)){
x=gwrrp$SDF$pred[(7*(i-1)+1):(7*(i-1)+7)]
xx=gwrrp$SDF$pred.se[(7*(i-1)+1):(7*(i-1)+7)]
y=rpvec[(7*(i-1)+1):(7*(i-1)+7)]
pdf(file=filename[i],paper="special",width=6, height=6)
plot(x,type="l",lwd=4,ylim=c(-70,80),xlab="",ylab="",main=cityname[i],xaxt="n",yaxt="n",cex.main=1.4)
xtx=c(1,2,3,4,5,6,7)
axis(1, xtx, labels = c("Apr","","Jun","","Aug","","Oct"),lwd=1,cex.axis=1.3) 
axis(2,lwd=1,cex.axis=1.3) 
lines(x+2*xx,lty=3,lwd=3)
lines(x-2*xx,lty=3,lwd=3)
points(y,lwd=4,col="dimgray")
dev.off()}

#example Leipzig
#############################################################################
setwd('C:/.../Quantlet/output')
leip<-read.csv("scores/Leipzig.csv")
lat1<-c(rep(52.3,times=7),rep(41.28,times=7),rep(52.52,times=7),
rep(51.45,times=7),rep(51.48,times=7),rep(40.45,times=7),
rep(48.73,times=7),rep(41.78,times=7),rep(51.32,times=7))

lon1<-c(rep(4.77,times=7),rep(2.07,times=7),rep(13.42,times=7),
rep(7.02,times=7),rep(-0.45,times=7),rep(-3.55,times=7),
rep(2.4,times=7),rep(12.58,times=7),rep(12.42,times=7))

install.packages("sp")
library(sp)

scores1<-rbind(scores[1:56,],leip)
coordinates(scores1)<-cbind(lat1,lon1)
gwrrp1<-gwr(rpvec~scores$V1+scores$V2+scores$V3+dummy1+dummy2,data=cbind(rpvec,scores[,2:4],dummy1,dummy2),
coords=cbind(lat,lon),bandwidth=bw,prediction=TRUE,se.fit=TRUE,hatmatrix=TRUE,
fit.points=scores1)
print(gwrrp1)
beta_0=c(beta_0, unique(gwrrp1$SDF[[2]])[9])
beta_1=c(beta_1, unique(gwrrp1$SDF[[3]])[9])
beta_2=c(beta_2, unique(gwrrp1$SDF[[4]])[9])
beta_3=c(beta_3, unique(gwrrp1$SDF[[5]])[9])
beta_4=c(beta_4, unique(gwrrp1$SDF[[6]])[9])
beta_5=c(beta_5, unique(gwrrp1$SDF[[7]])[9])
r2=c(r2, NA)
cityname<-c("Amsterdam","Barcelona","Berlin","Essen","London","Madrid","Paris","Rome","Stockholm","Leipzig")
results=data.frame(cityname,beta_0,beta_1,beta_2,beta_3,beta_4,beta_5,r2)
print(results)

rpl<-gwrrp1$SDF[[14]][57:63]#predicted rp for Leipzig
rp.se<-gwrrp$SDF[[16]]#rp se from old model
num<-c(3,4)#numbers of cities to compare to
x<-matrix(NA,length(num),7)
xx<-matrix(NA,length(num),7)
for (i in 1:length(num)){
x[i,]=rp[(7*(num[i]-1)+1):(7*(num[i]-1)+7)]
xx[i,]=rp.se[(7*(num[i]-1)+1):(7*(num[i]-1)+7)]
}
setwd('C:/.../Quantlet/figures')
	pdf(file="rple.pdf",paper="special",width=6, height=6)
plot(x[1,],type="l",lty=1,lwd=3,ylim=c(-51,51),xlab="",ylab="",
main="",xaxt="n",yaxt="n",cex.main=1.4,col="dimgray")
xtx=c(1,2,3,4,5,6,7)
axis(1, xtx, labels = c("Apr","","Jun","","Aug","","Oct"),lwd=1,cex.axis=1.3) 
axis(2,lwd=1,cex.axis=1.3) 
#
#lines(x[2,],lty=3,lwd=4)

lines(rpl,lty=1,lwd=4)

lines(x[1,]+2*xx[1,],lty=2,lwd=3,col="dimgray")
lines(x[1,]-2*xx[1,],lty=2,lwd=3,col="dimgray")
dev.off()
install.packages("msm")
library(msm)
setwd('C:/.../Quantlet/output')
dates1=(array( c(
    20040401, 20040501, 20040601, 20040701, 20040801, 20040901, 20041001,
    20050401, 20050501, 20050601, 20050701, 20050801, 20050901, 20051001,
    20060401, 20060501, 20060601, 20060701, 20060801, 20060901, 20061001,
    20070401, 20070501, 20070601, 20070701, 20070801, 20070901, 20071001,
    20080401, 20080501, 20080601, 20080701, 20080801, 20080901, 20081001,
    20090401, 20090501, 20090601, 20090701, 20090801, 20090901, 20091001,
    20100401, 20100501, 20100601, 20100701, 20100801, 20100901, 20101001),dim=c(7,7)))
dates2=(array( c(
    20040430, 20040531, 20040630, 20040731, 20040831, 20040930, 20041031,
    20050430, 20050531, 20050630, 20050731, 20050831, 20050930, 20051031,
    20060430, 20060531, 20060630, 20060731, 20060831, 20060930, 20061031,
    20070430, 20070531, 20070630, 20070731, 20070831, 20070930, 20071031,
    20080430, 20080531, 20080630, 20080731, 20080831, 20080930, 20081031,
    20090430, 20090531, 20090630, 20090731, 20090831, 20090930, 20091031,
    20100430, 20100531, 20100630, 20100731, 20100831, 20100930, 20101031),dim=c(7,7)))
price.hat=matrix(NA,nrow(dates1),ncol(dates1))
Aest<-read.csv("clei.csv")
Aest<-Aest[,2]
alpha1=3-Aest[1];alpha2=2*alpha1-Aest[2]-3;alpha3=alpha2+1-alpha1-Aest[3];
	A=cbind(c(0,0,-alpha3),c(1,0,-alpha2),c(0,1,-alpha1)) 
	ip<-diag(1,3,3);
	ep=c(0, 0, 1);
	e1=c(1, 0, 0);
tempfor<-read.csv("tempforLei.csv")
	tempfor<-tempfor[,2:5]
for (i in 1:ncol(dates1))
	{
		for (j in 1:nrow(dates1))
		{
		beg=dates1[j,i]
		en=dates2[j,i]
		it=which(tempfor[,1]==beg)
		itau1=it
		itau2=which(tempfor[,1]==en)
		if (length(itau2)==0){price.hat[j,i]=NA} else
 		{season=sum(tempfor[itau1:itau2, 3])
			if (ncol(A)==3)
			{
			x=c(tempfor[it-3,4], tempfor[it-2,4], tempfor[it-1,4])
			}
			if (ncol(A)==4)
			{
			x=c(tempfor[it-4,4], tempfor[it-3,4], tempfor[it-2,4], tempfor[it-1,4])
			}
			if (ncol(A)==7)
			{
			x=c(tempfor[it-7,4], tempfor[it-6,4], tempfor[it-5,4], tempfor[it-4,4], tempfor[it-3,4],
			tempfor[it-2,4], tempfor[it-1,4])
			}
			ax=e1%*%solve(A)%*%(MatrixExp(A*(itau2-itau1+1))-diag(1,nrow(A),ncol(A)))%*%x
			price.hat[j,i]=season+ax+rpl[j]
		}}}
print("estimated prices for Leipzig CAT futures")
dimnames(price.hat)<-list(month.abb[4:10], 2004:2010)
price.hat
#############################################################################
